qa_routes_file = "#{Rails.root}/lib/qa_routes.rb"
if File.exist?(qa_routes_file)
  require 'qa_routes.rb'
end

ActionController::Routing::Routes.draw do |map|

  map.connect        '/auth/saml',          :controller => 'saml', :action => 'new',      :conditions => {:method => :get}
  map.saml_select    '/auth/saml/select',   :controller => 'saml', :action => 'index',    :conditions => {:method => :get}
  map.saml_login     '/auth/saml/login',    :controller => 'saml', :action => 'new',      :conditions => {:method => :get}
  map.saml_callback  '/auth/saml/callback', :controller => 'saml', :action => 'create',   :conditions => {:method => :post}
  map.saml_metadata  '/auth/saml/metadata/:id', :controller => 'saml', :action => 'metadata', :conditions => {:method => :get}
  map.saml_test      '/auth/saml/test/:id', :controller => 'saml', :action => 'test', :conditions => {:method => :get}
  map.logged_out     '/logged_out', :controller => 'login', :action => 'logged_out', :conditions => {:method => :get}

  map.oauth_token '/auth/oauth2/token', :controller => 'oauth', :action => 'create', :conditions => {:method => :post}

  map.enable_connect '/enable_connect', controller: 'login', action: 'enable_connect', conditions: { method: :post }
  map.disable_connect '/disable_connect', controller: 'login', action: 'disable_connect', conditions: { method: :post }

  # API
  # FASPEX 3.0 is spread throughout different controllers. For example, the
  # /send endpoint is handled by the send controller. (But the send controller
  # also handles UI stuff.)
  # FASPEX OVERLAY ENDPOINTS

  map.namespace :api do |api|
    api.namespace :admin do |admin|
      #
      # =====================================================
      #
      # Workgroup and dropbox management (admin only)
      #
      admin.resources :workgroups, controller: 'workgroups'
      admin.resources :dropboxes,  controller: 'workgroups'
    end

    #
    # Server configuration
    #
    api.connect 'server_configuration',
                :controller => 'server_configuration',
                :action => 'server_configuration',
                :conditions => {:method => :get}

    #
    # Create package
    #
    api.connect 'users/:user_id/packages',
                :controller => 'packages',
                :action => 'create',
                :conditions => {:method => :post}
    #
    # Update package
    #
    api.connect 'users/:user_id/packages/:package_id',
                :controller => 'packages',
                :action => 'update',
                :conditions => {:method => :put}
    #
    # Patch package (don't require full spec)
    #
    api.connect 'users/:user_id/packages/:package_id/patch',
                :controller => 'packages',
                :action => 'patch',
                :conditions => {:method => :put}
    #
    # Delete package
    #
    api.connect 'users/:user_id/packages/:package_id',
               :controller => 'packages',
               :action => 'delete',
               :conditions => {:method => :delete}

    #
    # Bulk delete
    #
    api.connect 'users/:user_id/packages/',
                :controller => 'packages',
                :action => 'bulk_delete',
                :conditions => {:method => :delete}
    #
    # Get a package
    #
    api.connect 'users/:user_id/packages/:package_id',
                :controller => 'packages',
                :action => 'show',
                :conditions => {:method => :get}
    #
    # Search all packages for user
    #
    api.connect 'users/:user_id/packages',
                :controller => 'packages',
                :action => 'search',
                :conditions => {:method => :get}
    #
    # Forward a package
    #
    api.connect 'users/:user_id/packages/:package_id',
                :controller => 'packages',
                :action => 'forward',
                :conditions => {:method => :post}

    #
    # Search packages
    #
    api.connect 'packages',
                :controller => 'packages',
                :action => 'search_all',
                :conditions => {:method => :get}

    api.connect 'packages',
                :controller => 'packages',
                :action => 'search_all',
                :conditions => {:method => :post}

    #
    # Recalculate package stats
    #
    api.connect 'users/:user_id/packages/:package_id/recalculate',
      :controller => 'packages',
      :action => 'recalculate',
      :conditions => {:method => :post}

    #
    # Replicate package contents
    #
    api.connect 'users/:user_id/packages/:package_id/replicate_contents',
                :controller => 'packages',
                :action => 'replicate_contents',
                :conditions => {:method => :post}

    #
    # =====================================================
    #
    # Package upload / download initiation
    #
    api.connect 'users/:user_id/packages/:package_id/transfer_specs',
                :controller => 'transfer_specs',
                :action => 'get_transfer_spec',
                :conditions => {:method => :post}

    api.connect 'users/:user_id/packages/:package_id/issue_token',
                :controller => 'transfer_specs',
                :action => 'issue_token',
                :conditions => {:method => :post}

    #
    # =====================================================
    #
    # Transfer history / search
    #
    api.connect 'users/:user_id/packages/:package_id/transfers',
                :controller => 'transfers',
                :action => 'search',
                :conditions => {:method => :get}
    #
    # Remote / node-to-node transfer
    #
    api.connect 'users/:user_id/packages/:package_id/transfers',
                :controller => 'transfers',
                :action => 'remote_transfer',
                :conditions => {:method => :post}
    #
    # =====================================================
    #
    # View package contents
    #
    api.connect 'users/:user_id/packages/:package_id/files',
                :controller => 'files',
                :action => 'browse',
                :conditions => {:method => :post}

    #
    # =====================================================
    #
    # Delete files
    #
    api.connect 'users/:user_id/packages/:package_id/delete_contents',
                :controller => 'files',
                :action => 'delete_contents',
                :conditions => {:method => :post}

    #
    # =====================================================
    #
    # Rename files
    #
    api.connect 'users/:user_id/packages/:package_id/rename_contents',
                :controller => 'files',
                :action => 'rename_contents',
                :conditions => {:method => :post}

    #
    # =====================================================
    #
    # Me (current user)
    #
    api.connect 'users/me',
                :controller => 'me',
                :action => 'show',
                :conditions => {:method => :get}
    api.connect 'users/me/configuration',
                :controller => 'me',
                :action => 'configuration',
                :conditions => {:method => :get}
    #
    # =====================================================
    #
    # User information (admin only)
    #
    api.connect 'users/:user_id',
                :controller => 'users',
                :action => 'show',
                :conditions => {:method => :get}
    api.connect 'users/:user_id',
                :controller => 'users',
                :action => 'update',
                :conditions => {:method => :put}
    api.connect 'users',
                :controller => 'users',
                :action => 'index',
                :conditions => {:method => :get}
    api.connect 'users',
                :controller => 'users',
                :action => 'create',
                :conditions => {:method => :post}
    api.connect 'users/:user_id',
                :controller => 'users',
                :action => 'delete',
                :conditions => {:method => :delete}
    api.connect 'users/:user_id/configuration',
                :controller => 'users',
                :action => 'configuration',
                :conditions => {:method => :get}

    #
    # =====================================================
    #
    # User profile fields
    #

    api.resources :user_profile_fields, controller: 'user_profile_fields', only: [:index, :show]

    #
    # =====================================================
    #
    # Workgroup and dropbox management (admin only)
    #
    api.resources :workgroups, controller: 'workgroups', only: [:show, :index]
    api.resources :dropboxes,  controller: 'workgroups', only: [:show, :index]

    # Workgroup and dropbox memberships (admin, workgroup admin only)
    api.resources :workgroup_memberships, controller: 'workgroup_memberships'
    api.resources :dropbox_memberships,   controller: 'workgroup_memberships'


    #
    # =====================================================
    #
    # Metadata schema info
    #

    api.connect   'metadata_profiles/default_profile',
                  :controller => 'metadata_profiles',
                  :action => 'default_profile',
                  :conditions => {:method => :get}
    api.resources :metadata_profiles, only: [:index, :show]
    api.connect   'metadata_profiles/:id/validate_metadata',
                  :controller => 'metadata_profiles',
                  :action => 'validate_metadata',
                  :conditions => {:method => :post}

    #
    # =====================================================
    #
    # SAML configuration info
    #

    api.connect 'saml_configurations/default',
                :controller => 'saml_configurations',
                :action => 'default_configuration',
                :conditions => {:method => :get}
    api.resources :saml_configurations, controller: 'saml_configurations', only: [:index, :show]

    #
    # =====================================================
    #
    # Node information (admin only)
    #
    api.connect 'nodes/:node_id',
                :controller => 'nodes',
                :action => 'show',
                :conditions => {:method => :get}
    api.connect 'nodes',
                :controller => 'nodes',
                :action => 'index',
                :conditions => {:method => :get}
    #
    # =====================================================
    #
    # Share information (admin only)
    #
    api.connect 'nodes/:node_id/shares/:share_id',
                :controller => 'shares',
                :action => 'show',
                :conditions => {:method => :get}
    api.connect 'nodes/:node_id/shares',
                :controller => 'shares',
                :action => 'index',
                :conditions => {:method => :get}
  end

  # FASPEX 2.0
  # Send to service
  map.connect 'send-to',
      :controller => 'rest_api_services',
      :action => 'send_from_client',
      :conditions => {:method => :post}
  # Address-book service
  map.connect 'address-book',
      :controller => 'rest_api_services',
      :action => 'address_book_portable_contacts',
      :conditions => {:method => :get}
  map.connect 'address-book/:pc_path1',
      :controller => 'rest_api_services',
      :action => 'address_book_portable_contacts', :pc_path2 => '',
      :conditions => {:method => :get}
  map.connect 'address-book/:pc_path1/:pc_path2',
      :controller => 'rest_api_services',
      :action => 'address_book_portable_contacts',
      :conditions => {:method => :get}
  map.connect 'address-book/:pc_path1/:pc_path2/:requested_contact_id',
      :controller => 'rest_api_services',
      :action => 'address_book_portable_contacts',
      :conditions => {:method => :get}
  # Inbox service
  map.connect 'inbox.atom',
      :controller => 'rest_api_services',
      :action => 'package_feed',
      :box => 'inbox.atom', :format => 'xml',
      :conditions => {:method => :get}
  map.connect 'archive.atom',
      :controller => 'rest_api_services',
      :action => 'package_feed',
      :box => 'archive.atom', :format => 'xml',
      :conditions => {:method => :get}
  map.connect 'sent.atom',
      :controller => 'rest_api_services',
      :action => 'package_feed',
      :box => 'sent.atom', :format => 'xml',
      :conditions => {:method => :get}
  # Token issuing service
  map.connect 'issue-token',
      :controller => 'rest_api_services',
      :action => 'issue_token',
      :conditions => {:method => :post}
  # Signon service
  map.connect 'signon',
      :controller => 'rest_api_services',
      :action => 'signon',
      :conditions => {:method => :post}
  map.connect 'me',
      :controller => 'rest_api_services',
      :action => 'me',
      :conditions => {:method => :get}

  map.root :controller => :login, :action => :new
  map.resource :login,
               :controller => :login,
               :only => [:new, :create, :destroy, :show],
               :member => {
                   :authenticate_ds_user => :post
               }
  map.connect :health_check,
              controller: :login,
              action: :health_check,
              method: :get
  map.connect :health_check_light,
              controller: :login,
              action: :health_check_light,
              method: :get
  map.saml_logout '/saml_logout', :controller => :login, :action => :saml_logout

  map.logout '/logout', :controller => :login, :action => 'destroy'
  map.saml_options '/saml_options', :controller => :login, :action => 'saml_options'

  # legacy path support
  map.old_login 'login/login', :controller => :login, :action => :old_login

  map.resource :credits, :only => [:show]

  map.resources :registrations,
                :only => [:new, :create],
                :member => {
                  :confirmation => :get,
                }
  map.register '/register', :controller => :registrations, :action => 'new'

  map.resources :reset_passwords, :controller => :reset_password_tokens, :only => [:new, :create, :edit, :update]

  map.namespace :settings do |settings|
    settings.resource :preferences, :only => [:edit, :update]
    settings.resource :password, :only => [:edit, :update]
    settings.resources :contacts, :only => [:index, :destroy]
    settings.resources :distribution_lists, only: [:index, :create, :new, :destroy], collection: { batch_update: :put }
    settings.resources :sender_quota, :only => [:index]
  end

  map.namespace :workgroup do |ns|
    ns.resources :workgroups,
        :except => :none,
        :path_prefix => nil,
        :name_prefix => nil,
        :member => {
          :auto_complete_for_user_name => :post,
          :ajax_update_members            => :post
        } do |workgroup|
      workgroup.resources :deliveries,
          :only => [:index, :show],
          :collection => {
            :auto_complete_for_recipient_list => :post,
            :ajax_update_wg_packages_list => :post,
            :ajax_retry_all_relays => :post,
            :history => :get
          },
          :member => {
            :ajax_update_package => :post,
            :remove_from_view => :post,
            :ajax_update_file_browser => :post,
            :ajax_retry_package_upload => :post
          } do |delivery|
        delivery.resources :file_relay_requests,
            only: :index,
            collection: { ajax_update_index: :post },
            member: { ajax_retry_relay: :post }
      end
      workgroup.resources :users,
          :except => :show
    end
  end
  # legacy URL support
  map.connect 'workgroups/:workgroup_id/packages/:id', :controller => 'workgroup/deliveries',
              :action => :show, :method => :get

  map.entitlement 'entitlement', :controller => 'entitlement', :action => 'login', :method => :get
  map.namespace :dropbox do |ns|
    ns.resources :dropboxes,
                 :except => :none,
                 :path_prefix => nil,
                 :name_prefix => nil,
                 :member => {
                   :auto_complete_for_user_name => :post,
                   :ajax_update_members => :post } do |dropbox|
      dropbox.resources :deliveries,
                        :only => [:new, :create, :index, :show],
                        :collection => {
                            :auto_complete_for_recipient_list => :post,
                            :ajax_update_wg_packages_list => :post,
                            :ajax_retry_all_relays => :post,
                            :history => :get
                        },
                        :member => {
                            :ajax_update_package => :post,
                            :remove_from_view => :post,
                            :ajax_stop_upload => :post,
                            :ajax_update_file_browser => :post,
                            :ajax_retry_package_upload => :post,
                            :delete_package => :post
                        }
      dropbox.resources :users,
                        :except => :show,
                        :collection => {
                            :invite => :get,
                            :create_invitation => :post
                        },
                        :member => {
                            :resend_invitation => :post,
                            :access_url => :get
                        }
    end
  end

  map.namespace :external do |external|

    external.resources :user_deliveries, only: [], member: { download: :post }
    external.resources :submissions, only: [:new, :create],
                       :collection => {
                         :find => :get
                       }
    external.resources :dropbox_submissions, only: [:new, :create]
    external.resource :to, :controller => :to, only: [:new, :create], :member => { :confirmation => :get, }
    external.resource :submission_status,
                      :controller => :submission_status,
                      :only => :show,
                      :member => {
                          :ajax_update => :post,
                          :ajax_retry_package_upload => :post
                      }

    external.forbidden_error 'forbidden', controller: 'error', action: 'forbidden'
  end

  map.namespace :account do |account|
    account.resource :template_user, :only => [:edit, :update] do |template_user|
      template_user.resources :memberships, :only => [:index],
        :member => {
          :remove => :post
        },
        :collection => {
          :add => :post
        }
    end
    account.resources :directory_service_groups,
      :collection => {
        :bulk_action => :post
      },
      :member => {
        :destroy_membership => :post
      }
    account.resources :saml_groups,
      :collection => {
        :bulk_action => :post
      },
      :member => {
        :destroy_membership => :post
      }
    account.resources :directory_service_users, :only => [:index], collection: { import: :post }
    account.resources :registrations, :only => [:index],
      :collection => {
        :bulk_action => :post,
      }

    account.external_users :external_users, controller: :users, action: :external_users
    account.resources :users,
      :only => [:index, :new, :edit, :create, :update],
      :collection => {
        :ajax_update_user_ad_history => :post,
        :bulk_action => :post
      },
      :member => {
        :edit_wg_memberships => :get,
        :set_admin => :post,
        :delete_membership => :post,
        :create_membership => :post
      }

    account.resources :local_users, :only => [] do |local_users|
      local_users.resource :password,
                           :only => [:edit, :update, :reset_password],
                           :collection => {
                               :reset_password => :get
                           }
    end
  end

  map.resources :contacts, :only => :index
  map.resources :cc_contacts, only: :index
  map.resources :user_profile_fields,
                only: [:index, :ajax_add_user_profile_field],
                collection: {
                    ajax_add_user_profile_field: :get,
                    update_all: :post
                }

  map.resources :metadata_profiles,
                :except => :show,
                :collection => { :update_default => :post }

  map.resources :received,
                :controller => :received,
                :only => [:index, :show],
                :collection => {
                    :auto_complete_for_recipient_list => :post,
                    :ajax_populate_address_book => :post,
                    :ajax_remove_contacts => :post,
                    :ajax_update_received_list => :post,
                    :clean_up => :post,
                    :history => :get
                },
                :member => {
                    :remove_from_view => :post,
                    :ajax_update_package => :post,
                    :ajax_update_file_browser => :post
                }
  # support for links in old emails
  map.connect 'received/show/:id', :controller => :received, :action => :show
  # support for mobile links in emails
  map.mobile_package_received 'package/access/aspera-faspex/:id', :controller => :received, :action => :show

  map.resources :sent,
                :controller => :sent,
                :only => [:index, :show],
                :collection => {
                    :auto_complete_for_recipient_list => :post,
                    :ajax_populate_address_book => :post,
                    :ajax_remove_contacts => :post,
                    :ajax_update_sent_list => :post,
                    :clean_up => :post,
                    :history => :get,
                    :mark_package_queued => :post
                },
                :member => {
                    :remove_from_view => :post,
                    :ajax_update_package => :post,
                    :ajax_stop_upload => :post,
                    :ajax_retry_package_upload => :post,
                    :show_details => :get,
                    :ajax_update_file_browser => :post,
                    :delete_package => :post,
                    :retry_relay => :get
                }

  map.resources :send,
      :controller => :send,
      :only => [:new, :create]

  map.resources :pending,
       :controller => :pending,
       :only => [:index, :show],
      :collection => {
          :auto_complete_for_recipient_list => :post,
          :ajax_populate_address_book => :post,
          :ajax_remove_contacts => :post,
          :ajax_update_sent_list => :post,
          :clean_up => :post,
          :history => :get
      },
      :member => {
          :remove_from_view => :post,
          :ajax_update_package => :post,
          :ajax_stop_upload => :post,
          :ajax_retry_package_upload => :post,
          :show_details => :get,
          :ajax_update_file_browser => :post,
          :ajax_edit_release_date => :post,
          :ajax_release_now => :post,
          :delete_package => :post
      }

  map.namespace :admin do |admin|
    map.test_admin_node '/admin/nodes/test/:id', method: :post, controller: 'admin/nodes', action: 'test', id: nil
    admin.resources :nodes, :except => [:show], member: { :browse => :get } do |node|
      node.resources :shares, :except => [:show, :index], :collection => { :ajax_get_shares => :get }
    end
    admin.resources :shares, collection: { batch_update: :put }, member: { search: :get, add_member: :post }
    admin.resources :share_memberships, only: [:destroy]
    admin.resources :authorization_domains, except: [:show], member: { ajax_test_ad_settings: :post }

    admin.resources :saml_configurations, :controller => :saml_configurations,
                    :member => {
                        :update           => :put,
                        :login            => :get,
                        :toggle_default   => :get,
                        :toggle_enabled   => :get,
                    },
                    :collection => {
                        :create => :post,
                        :ajax_add_user_profile_field => :get,
                        :ajax_get_saml_settings_from_metadata => :get,
                        :ajax_get_saml_settings_from_file => :post,
                    }

    admin.resources :post_processes, :except => [:show],
                :member => {
                    :show_events => :get,
                    :show_event_log => :get,
                    :activate => :post,
                    :inactivate => :post,
                    :delete => :post
                }

    admin.resources :packages,
                :only => [:index, :show],
                :collection => {
                    :ajax_populate_address_book => :post,
                    :ajax_update_delivery_list => :post,
                    :goto_delete_expired => :post,
                    :delete_expired => :get,
                    :confirm_delete_expired => :post,
                    :ajax_update_relay_list => :post,
                    :relays => :get
                },
                :member => {
                    :show_details => :get,
                    :delete_files => :post,
                    :ajax_update_package => :post,
                    :relay_details => :get,
                    :ajax_update_relay_details => :post,
                    :ajax_retry_package_relay => :post,
                    :ajax_cancel_package_relay => :post,
                    :ajax_update_file_browser => :post
                }



    admin.namespace :notifications do |notifications|
      notifications.resource :login_announcement,   :only => [:edit, :update]
      notifications.resource :package_instructions, :only => [:edit, :update]
    end

    admin.namespace :settings do |settings|
      settings.resource :web_server,               :only => [:edit, :update, :ajax_add_alternate_server_address], controller: 'web_server', collection: {
          ajax_add_alternate_server_address: :get,
      }
      settings.resource :transfer_options,         :only => [:edit, :update]
      settings.resource :security,                 :only => [:edit, :update], controller: 'security'
      settings.resource :web_server_security,      :only => [:edit, :update], controller: 'web_server_security'
      settings.resource :transfer_server_security, :only => [:edit, :update], controller: 'transfer_server_security'
      settings.resource :package_storage,          :only => [:edit, :update], controller: 'package_storage'
      settings.resource :display_settings,         :only => [:edit, :update, :upload_logo, :remove_logo], :collection => {:remove_logo => :post}
      settings.resource :save_restore,             :only => [:index], :member => {:save => :get, :restore => :post, :index => :get}, controller: 'save_restore_config'
      settings.resource :license,                  :only => [:edit, :update], controller: 'license'
      settings.resource :entitlement,              :only => [:edit, :update], controller: 'entitlement'
      settings.resources :global_distribution_lists, only: [:index, :create, :new, :destroy], collection: { batch_update: :put }
    end
  end

  map.resources :source_shares, :member => {
    :browse      => :get,
    :browse_more => :get
  }

  map.resources :file_sources, controller: :source_shares, only: [:index]

  map.resource :m_configuration,
               :controller => :m_configuration,
               :only => [:edit, :update]

  map.resources :m_templates,
                :controller => :m_template,
                :only => [:show, :update, :edit],
                :member => {
                    :wizard => :get,
                    :show_frame => :get
                }


  #map.resource :ms_admin,
  #             :controller => :ms_admin,
  #             :only => [:edit, :update],
  #             :member => {
  #                 :ajax_update_home_server_list => :post,
  #                 :new_server => :get,
  #                 :edit_server => :get,
  #                 :create_server => :post,
  #                 :update_server => :put,
  #                 :convert_to_slave => :get,
  #                 :set_role => :put,
  #                 :sync_history => :get,
  #                 :delete_server => :post,
  #                 :execute_sync => :post,
  #                 :force_full_sync => :post,
  #                 :ajax_multi_server_test => :post,
  #                 :list_rules => :get,
  #                 :new_rule => :get,
  #                 :edit_rule => :get,
  #                 :create_rule => :post,
  #                 :delete_rule => :post,
  #                 :update_rule => :put,
  #                 :update_server_rules => :post,
  #                 :recalc_homeservers => :get,
  #                 :ajax_update_home_server_recalc => :post,
  #                 :reset_configuration => :post
  #             }

  map.resources :external_deliveries,
                :controller => :external_delivery,
                :only => :show,
                :member => {
                    :error => :get,
                    :ajax_update_package => :post,
                    :ajax_update_file_browser => :post
                }
  # support for legacy external delivery links to keep working
  map.connect 'external_delivery/show/:id', :controller => :external_delivery, :action => :show

  map.resource :ad_admin,
               :controller => :ad_admin,
               :only => :none,
               :member => {
                   :edit_config => :get,
                   :ajax_test_ad_settings => :post,
                   :update_config => :post,
                   :operation_history => :get,
                   :view_operation_summary => :get
               }

  map.resource :remote_admin,
      :controller => :remote_admin,
      :member => {
         :multi_server_test => :get,
         :query_journal_index => :get,
         :process_payload => :post
      },
      :only => :none do |ra|
    ra.resources :users,
        :controller => 'remote_admin/workgroup_users',
        :only => [:create, :update, :destroy],
        :requirements => { :id => /.*/ }, # to allow for usernames with dots
        :path_prefix => '/remote_admin/workgroups/:wg_name',
        :name_prefix => 'remote_admin_workgroup_',
        :collection => { :add => :post, :remove => :post, :create_invitation => :post, :set_access_level => :post},
        :member => { :resend_invitation => :post, :update_membership => :put }
    ra.resources :ds_groups,
        :controller => 'remote_admin/workgroup_ds_groups',
        :only => :none,
        :path_prefix => '/remote_admin/workgroups/:wg_name',
        :name_prefix => 'remote_admin_workgroup_',
        :collection => { :add => :post, :remove => :post, :set_access_level => :post }
    ra.resources :registrations,
        :controller => 'remote_admin/registrations',
        :only => [:create]
    ra.resources :invitations,
        :controller => 'remote_admin/invitations',
        :only => [:create, :destroy, :update],
        :requirements => { :id => /.*/ }, # to allow for email addresses with dots
        :member => {:resend => :post},
        :collection => {:create_from_public_url => :post}
    ra.resources :packages,
        :controller => 'remote_admin/packages',
        :only => :none,
        :member => {:mark_for_deletion => :post}
  end

  map.resource :package_relay, :controller => :package_relay, :only => :none,
               :member => {:relay_package => :post, :relay_access => :post}

  map.resources :invitations, member: { :resend => :post, :access_url => :get }
  map.resources :deliveries, member: { download: :post, forward: :get, forward_submit: :post }
  map.resources :user_deliveries, member: { download: :post }
  map.resources :workgroup_deliveries, member: { download: :post }

  map.public_user_url 'send/to/user/:name', controller: 'external/to', action: 'new',    conditions: { method: :get },  type: 'user'
  map.public_user_url 'send/to/user/:name', controller: 'external/to', action: 'create', conditions: { method: :post }, type: 'user'

  map.public_dropbox_url 'send/to/dropbox/:name', controller: 'external/to', action: 'new',    conditions: { method: :get },  type: 'dropbox'
  map.public_dropbox_url 'send/to/dropbox/:name', controller: 'external/to', action: 'create', conditions: { method: :post }, type: 'dropbox'

  map.public_user_legacy_url 'send/to/:name',     controller: 'external/to_legacy', action: 'show', conditions: { method: :get }, type: 'user_or_dropbox'
  map.public_user_legacy_url 'send/to/users/:id', controller: 'external/to_legacy', action: 'show', conditions: { method: :get }, type: 'user'
  map.public_user_legacy_url 'send/to/dropboxes/:id', controller: 'external/to_legacy', action: 'show', conditions: { method: :get }, type: 'dropbox'

  # named routes for different error conditions
  map.show_error 'error/show_error', :controller => :e_error, :action => :show_error
  map.page_not_found 'error/page_not_found', :controller => :e_error, :action => :page_not_found
  map.gone 'error/gone', :controller => :e_error, :action => :gone
  map.expired_or_invalid 'error/expired_or_invalid', :controller => :e_error, :action => :expired_or_invalid
  map.access_denied 'error/access_denied', :controller => :e_error, :action => :access_denied
  map.not_implemented 'error/not_implemented', :controller => :e_error, :action => :not_implemented
  map.internal_server_error 'error/500', :controller => :e_error, :action => :internal_server_error

  if Rails.env.production?
    # Catch-all so we can gracefully handle badly-formed requests
    map.connect "*anything.xml",
    :controller => "e_error",
    :action => "unknown_url",
    :format => 'xml'

    # Catch-all so we can gracefully handle badly-formed requests
    map.connect "*anything" ,
    :controller => "e_error" ,
    :action => "unknown_url"
  end
end
