-- Migration from statscollector/branches/1.0 (faspex)

USE /*DBNAME*/`faspex`/*DBNAME*/;

-- fasp_files does not exist on all installation
-- Create an empty table to be able to proceed
CREATE TABLE IF NOT EXISTS `fasp_files` (
  `node_uuid` varchar(36) NOT NULL,
  `session_id` varchar(36) NOT NULL,
  `status` varchar(16) NOT NULL,
  `started_at` datetime default NULL,
  `stopped_at` datetime default NULL,
  `file_fullpath` text NOT NULL,
  `file_index` varchar(32) NOT NULL,
  `file_basename` text NOT NULL,
  `size` bigint(20) NOT NULL default '0',
  `start_byte` bigint(20) NOT NULL default '0',
  `bytes_written` bigint(20) NOT NULL default '0',
  `bytes_contig` bigint(20) NOT NULL default '0',
  `bytes_lost` bigint(20) NOT NULL default '0',
  `usecs` bigint(20) NOT NULL default '0',
  `checksum` varchar(255) default NULL,
  `checksum_type` varchar(16) default NULL,
  `err_code` int(11) NOT NULL,
  `err_desc` varchar(255) default NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

-- Rename the tables conflicting with the new schema
RENAME TABLE `fasp_files` TO `fasp_files_old`;
RENAME TABLE `fasp_sessions` TO `fasp_sessions_old`;
RENAME TABLE `fasp_nodes` TO `fasp_nodes_old`;

-- Create the new schema
AsperaImport sc_schema_v6.sql;

-- Copy with some adaptation the data to the new schema
SET @seq := 0;

INSERT INTO `fasp_files`
  (`seq_created`, `seq_modified`,
   `file_id`,
   `transfer_id`, `session_id`,
   `session_count`,
   `status`,
   `started_at`, `stopped_at`, `updated_at`,
   `client_path`,
   `server_path`,
   `size`, `start_byte`, `bytes_written`,
   `bytes_contig`, `bytes_lost`, `elapsed_usec`, `err_code`, `err_desc`,
   `checksum`,
   `checksum_type`,
   `bytes_processed`)
  SELECT
   (@seq := @seq + 1), (@seq := @seq + 1),
   CONCAT(IF(UPPER(fso.initiator)='LOCAL', 'CLIENT#', 'REMOTE#'), ffo.file_index),
   ffo.session_id, ffo.session_id,
   1,
   IF (UPPER(ffo.status) IN ('RUNNING','COMPLETED','ERROR','UNKNOWN'), UPPER(ffo.status), 'UNKNOWN'),
   IFNULL(ffo.started_at, NOW()), ffo.stopped_at, NOW(),
   IF(UPPER(fso.initiator)='LOCAL', ffo.file_fullpath, NULL),
   IF(UPPER(fso.initiator)='REMOTE', NULL, ffo.file_fullpath),
   ffo.size, ffo.start_byte, ffo.bytes_written,
   ffo.bytes_contig, ffo.bytes_lost, ffo.usecs, ffo.err_code, ffo.err_desc,
   ffo.checksum,
   IF(ISNULL(ffo.checksum_type), 'NONE', IF(UPPER(ffo.checksum_type) IN ('NONE','SHA1','MD5','UNKNOWN'), UPPER(ffo.checksum_type), 'UNKNOWN')),
   0
  FROM fasp_files_old ffo, fasp_sessions_old fso
  WHERE fso.node_uuid = ffo.node_uuid AND fso.session_id = ffo.session_id;

-- Create an entry for each session for which we have both side
SET @cur_stats_id := 0;

INSERT INTO `fasp_stats`
  (`seq_created`, `seq_modified`,
   `status`,
   `started_at`,
   `stopped_at`,
   `updated_at`,
   `transport`,

   `files_complete`,
   `files_failed`,
   `files_attempted`,
   `files_skipped`,

   `bytes_written`,
   `bytes_transferred`,
   `bytes_lost`,
   `elapsed_usec`,

   `err_code`,
   `err_desc`,
   `bytes_pretransfer`,
   `files_pretransfer`,
   `dirs_pretransfer`)
  SELECT
   (@seq := @seq + 1), (@seq := @seq + 1),
   IF(UPPER(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.status, fsoc.status)) IN ('RUNNING','PAUSED','COMPLETED','CANCELLED','ERROR','WILLRETRY','ORPHANED','UNKNOWN'), UPPER(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.status, fsoc.status)), 'UNKNOWN'),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.started_at, fsoc.started_at), NOW()),
   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.stopped_at, fsoc.stopped_at),
   NOW(),
   IF(UPPER(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.transport, fsoc.transport)) IN ('FASP2','HTTP','UNKNOWN'), UPPER(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.transport, fsoc.transport)), 'UNKNOWN'),

   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.files_complete, fsoc.files_complete),
   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.files_failed, fsoc.files_failed),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.files_attempted, fsoc.files_attempted), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.files_skipped, fsoc.files_skipped), 0),

   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.bytes_written, fsoc.bytes_written),
   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.bytes_transferred, fsoc.bytes_transferred),
   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.bytes_lost, fsoc.bytes_lost),
   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.usecs, fsoc.usecs),

   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.err_code, fsoc.err_code),
   IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.err_desc, fsoc.err_desc),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.bytes_pretransfer, fsoc.bytes_pretransfer), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.files_pretransfer, fsoc.files_pretransfer), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.dirs_pretransfer, fsoc.dirs_pretransfer), 0)
  FROM fasp_sessions_old fsoc, fasp_sessions_old fsos
  WHERE UPPER(fsoc.initiator) = 'LOCAL' AND UPPER(fsos.initiator) = 'REMOTE'
        AND fsoc.session_id = fsos.session_id
  ORDER BY fsoc.id ASC;

INSERT INTO `fasp_sessions`
  (`session_id`,
   `stats_id`,
   `transfer_id`,
   `server_node_id`,
   `server_ip`,
   `server_ssh_port`,
   `server_fasp_port`,
   `client_node_id`,
   `client_ip`,
   `client_fasp_port`,
   `args_attempted`,
   `args_completed`,
   `paths_attempted`,
   `paths_failed`,
   `paths_irreg`,
   `paths_excluded`,
   `dirscans_completed`,
   `filescans_completed`,
   `mkdirs_attempted`,
   `mkdirs_failed`,
   `mkdirs_passed`,
   `processing_complete`)
  SELECT
   fsoc.session_id,
   (@cur_stats_id := @cur_stats_id + 1),
   fsoc.session_id,
   fsos.node_uuid,
   IFNULL(fsos.server_addr, fsoc.server_addr),
   IF(ISNULL(fsos.server_addr), fsoc.server_sshport, fsos.server_sshport),
   IF(ISNULL(fsos.server_addr), fsoc.server_faspport, fsos.server_faspport),
   fsoc.node_uuid,
   IFNULL(fsos.client_addr, fsoc.client_addr),
   IF(ISNULL(fsos.client_addr), fsoc.client_faspport, fsos.client_faspport),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.args_attempted, fsoc.args_attempted), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.args_completed, fsoc.args_completed), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.paths_attempted, fsoc.paths_attempted), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.paths_failed, fsoc.paths_failed), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.paths_irreg, fsoc.paths_irreg), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.paths_excluded, fsoc.paths_excluded), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.dirscans_completed, fsoc.dirscans_completed), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.filescans_completed, fsoc.filescans_completed), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.mkdirs_attempted, fsoc.mkdirs_attempted), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.mkdirs_failed, fsoc.mkdirs_failed), 0),
   IFNULL(IF(fsos.bytes_transferred > fsoc.bytes_transferred, fsos.mkdirs_passed, fsoc.mkdirs_passed), 0),
   IFNULL(IFNULL(fsoc.processing_complete, fsos.processing_complete), false)
  FROM fasp_sessions_old fsoc, fasp_sessions_old fsos
  WHERE UPPER(fsoc.initiator) = 'LOCAL' AND UPPER(fsos.initiator) = 'REMOTE'
        AND fsoc.session_id = fsos.session_id
  ORDER BY fsoc.id ASC;

INSERT INTO `fasp_transfers`
  (`transfer_id`, `stats_id`, `reported_by`,
   `last_session_id`, `session_count`, `retry_timeout`,
   `client_cluster_id`, `server_cluster_id`,
   `server_host_name`, `user`,
   `operation`,
   `cipher`,
   `source_paths`,
   `dest_paths`,
   `cookie`, `tags`, `token`,
   `server_docroot`, `client_docroot`)
  SELECT
   fs.session_id, fs.stats_id, 'CLIENT_AND_SERVER',
   fs.session_id, 1, IFNULL(fsoc.retry_timeout, 0),
   fs.client_node_id, fs.server_node_id,
   null, fsoc.user,
   IF (UPPER(fsoc.operation) IN ('UPLOAD','DOWNLOAD','UNKNOWN'), UPPER(fsoc.operation), 'UNKNOWN'),
   IF (UPPER(fsoc.cipher) IN ('NONE','AES128','AES256','UNKNOWN'), UPPER(fsoc.cipher), 'UNKNOWN'),
   IFNULL(fsoc.source_paths, fsos.source_paths),
   IFNULL(fsoc.dest_path, fsos.dest_path),
   fsoc.cookie, null, fsoc.token,
   null, null
  FROM fasp_sessions_old fsoc, fasp_sessions_old fsos, fasp_sessions fs
  WHERE UPPER(fsoc.initiator) = 'LOCAL' AND UPPER(fsos.initiator) = 'REMOTE'
        AND fsoc.session_id = fsos.session_id AND fsoc.session_id = fs.session_id;

-- Create an entry for each session for which we have only one side
INSERT INTO `fasp_stats`
  (`seq_created`, `seq_modified`,
   `status`,
   `started_at`, `stopped_at`,`updated_at`,
   `transport`,
   `files_complete`, `files_failed`,
   `files_attempted`, `files_skipped`,
   `bytes_written`, `bytes_transferred`, `bytes_lost`, `elapsed_usec`,
   `err_code`, `err_desc`,
   `bytes_pretransfer`, `files_pretransfer`,
   `dirs_pretransfer`)
  SELECT
   (@seq := @seq + 1), (@seq := @seq + 1),
   IF(UPPER(fso.status) IN ('RUNNING','PAUSED','COMPLETED','CANCELLED','ERROR','WILLRETRY','ORPHANED','UNKNOWN'), UPPER(fso.status), 'UNKNOWN'),
   IFNULL(fso.started_at, NOW()), fso.stopped_at, NOW(),
   IF(UPPER(fso.transport) IN ('FASP2','HTTP','UNKNOWN'), UPPER(fso.transport), 'UNKNOWN'),
   fso.files_complete, fso.files_failed,
   IFNULL(fso.files_attempted, 0), IFNULL(fso.files_skipped, 0),
   fso.bytes_written, fso.bytes_transferred, fso.bytes_lost, fso.usecs,
   fso.err_code, fso.err_desc,
   IFNULL(fso.bytes_pretransfer, 0), IFNULL(fso.files_pretransfer, 0),
   IFNULL(fso.dirs_pretransfer, 0)
  FROM fasp_sessions_old fso
  WHERE NOT EXISTS(SELECT * FROM fasp_sessions fs WHERE fs.session_id = fso.session_id)
  ORDER BY fso.id ASC;

INSERT INTO `fasp_sessions`
  (`session_id`,
   `stats_id`,
   `transfer_id`,
   `server_node_id`,
   `server_ip`, `server_ssh_port`, `server_fasp_port`,
   `client_node_id`,
   `client_ip`, `client_fasp_port`,
   `args_attempted`, `args_completed`,
   `paths_attempted`, `paths_failed`,
   `paths_irreg`, `paths_excluded`,
   `dirscans_completed`, `filescans_completed`,
   `mkdirs_attempted`, `mkdirs_failed`,
   `mkdirs_passed`)
  SELECT
   fso.session_id,
   (@cur_stats_id := @cur_stats_id + 1),
   fso.session_id,
   IF(UPPER(fso.initiator)='LOCAL', NULL, fso.node_uuid),
   fso.server_addr, fso.server_sshport, fso.server_faspport,
   IF(UPPER(fso.initiator)='LOCAL', fso.node_uuid, NULL),
   fso.client_addr, fso.client_faspport,
   IFNULL(fso.args_attempted, 0), IFNULL(fso.args_completed, 0),
   IFNULL(fso.paths_attempted, 0), IFNULL(fso.paths_failed, 0),
   IFNULL(fso.paths_irreg, 0), IFNULL(fso.paths_excluded, 0),
   IFNULL(fso.dirscans_completed, 0), IFNULL(fso.filescans_completed, 0),
   IFNULL(fso.mkdirs_attempted, 0), IFNULL(fso.mkdirs_failed, 0),
   IFNULL(fso.mkdirs_passed, 0)
  FROM fasp_sessions_old fso
  WHERE NOT EXISTS(SELECT * FROM fasp_sessions fs WHERE fs.session_id = fso.session_id)
  ORDER BY fso.id ASC;

INSERT INTO `fasp_transfers`
  (`transfer_id`, `stats_id`,
   `reported_by`,
   `last_session_id`, `session_count`, `retry_timeout`,
   `client_cluster_id`, `server_cluster_id`,
   `server_host_name`, `user`,
   `operation`,
   `cipher`,
   `source_paths`,
   `dest_paths`,
   `cookie`, `tags`, `token`,
   `server_docroot`, `client_docroot`)
  SELECT
   fs.session_id, fs.stats_id,
   IF(UPPER(fso.initiator)='LOCAL', 'CLIENT', 'SERVER'),
   fs.session_id, 1, IFNULL(fso.retry_timeout, 0),
   fs.client_node_id, fs.server_node_id,
   null, fso.user,
   IF (UPPER(fso.operation) IN ('UPLOAD','DOWNLOAD','UNKNOWN'), UPPER(fso.operation), 'UNKNOWN'),
   IF (UPPER(fso.cipher) IN ('NONE','AES128','AES256','UNKNOWN'), UPPER(fso.cipher), 'UNKNOWN'),
   fso.source_paths, fso.dest_path,
   fso.cookie, null, fso.token,
   null, null
  FROM fasp_sessions_old fso, fasp_sessions fs
  WHERE NOT EXISTS(SELECT * FROM fasp_transfers ft WHERE ft.transfer_id = fs.session_id)
        AND fs.session_id = fso.session_id;

-- Save the next sequence number
UPDATE `sc_iterator_sequence` SET next = @seq+1;

-- sc_version will be updated by the Java code

-- Migrate fasp_nodes:
--   The table will be filled by the discovery class.
--   So we just need to drop the table and recreate with the
--   new definition
-- Nothing to do for the new tables: configuration and api_commands

-- Fill sc_nodes_poll_status with nodes info:
--   The data in fasp_nodes can't be migrated and are just removed.
--   So here we just need to remove the entries.
--   Note that the polling of the stats will restart from the first
--   session (because iteration token is reset) but it should not be a big
--   problem (except that some stats will be get again) because
--   only the new session/file will be finally inserted...
-- sc_nodes_poll_config will be filled by the Java code

-- DROP the old tables
DROP TABLE `fasp_files_old`;
DROP TABLE `fasp_sessions_old`;
DROP TABLE `fasp_nodes_old`;
